#ifndef __QUICK_TYPE_H
#define __QUICK_TYPE_H 1

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <System/SysEvtMgr.h>
#include <graffiti.h>

#define NON_PORTABLE
#include <Hardware/Hardware.h>
#include <System/Globals.h>

#include "quicktypeRsc.h"

#define qtKeyboardWidth     108
#define qtKeyboardHeight    55
#define qtKeyboardLeft      ( hwrDisplayWidth - qtKeyboardWidth ) / 2
#define qtKeyboardTop       hwrGraffitiTop
#define qtHorizBoxes        9
#define qtVertBoxes         5 
#define qtBoxWidth          qtKeyboardWidth / qtHorizBoxes
#define qtBoxHeight         qtKeyboardHeight / qtVertBoxes
#define qtFlickMin          4
#define qtFlickMax          15

#define qtMaxCalAdjust      10

#define qtPrevInfo          1010
#define qtPrefInfo          1011
#define ftrGDB              1012
#define ftrCurrentRecord    1013

#define prfPrefs            1

#define qtKeyUnrecognized  -1
#define qtKeyShift         -2
#define qtKeyCapsLock      -3
#define qtKeyAlphaKeyboard -4
#define qtKeyNumKeyboard   -5
#define qtKeyAlphaPunc     -6
#define qtKeyNumPunc       -7

#define XMASK              0x000000ff
#define YMASK              0x0000ff00
#define GOODMASK           0x00010000

typedef struct {
	Char primary;
	CharPtr left;
	CharPtr down;
} KeyEntry;

typedef struct {
	Boolean autoSpace;
	Boolean builtIn;
	SByte calX;
	SByte calY;
	Boolean graffiti;
} Prefs;

#define getX( l ) ( l & XMASK )
#define getY( l ) ( ( l & YMASK ) >> 8 )
#define setX( dest, x ) ( dest = ( dest & ~XMASK ) | getX( x ) )
#define setY( dest, y ) ( dest = ( dest & ~YMASK ) | ( getX( y ) << 8 ) )
#define getGood( l ) ( ( l & GOODMASK ) >> 16 )
#define setGood( dest, good ) ( dest = ( dest & ~GOODMASK ) | ( (UInt32) good << 16 ) )
#define getSpaceAfterPunc( l ) ( ( l & GOODMASK ) >> 16 )
#define setSpaceAfterPunc( dest, good ) ( dest = ( dest & ~GOODMASK ) | ( (UInt32) good << 16 ) )

#define getValue( source, r ) ( ( source >> ( r * 8 ) ) & XMASK )

inline Boolean isPossibleValidEvent( EventPtr event );
inline void handlePenMove( int x, int y, EventPtr event );
inline void handlePenDown( int x, int y, EventPtr event );
inline void handlePenUp( int x, int y, EventPtr event, Prefs* prefs );
inline DmOpenRef OpenOrCreateDB();

inline Boolean isQuickShift( Byte prevX, Byte prevY, int x, int y );
inline Boolean isQuickSpace( Byte prevX, Byte prevY, int x, int y );
inline Boolean isQuickLeft( Byte prevX, Byte prevY, int x, int y );
inline Boolean isQuickDown( Byte prevX, Byte prevY, int x, int y );

inline void processAlpha( WChar chr, Boolean quickShift, Boolean quickSpace, Boolean autoSpace );
inline void processString( Char* str, Boolean quickShift, Boolean quickSpace, Boolean autoSpace );
inline Boolean processNonAlpha( int ichr, EventPtr event, Prefs* prefs );
inline Boolean isPunc( int ichr );

void UnpackKeyEntry( KeyEntry* entry, const VoidPtr record );

#endif


