#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <Hardware/Hardware.h>

#include "quicktype.h"


static void ensureValid( Prefs* prefs );
static void StorePrefs();
static Prefs* GetPrefs();
static void SetCalView( Prefs* prefs );
static void Finished();


static Boolean HandleEvent( EventPtr event ) {
	FormPtr form = FrmGetActiveForm();
	ControlPtr autoSpaceChkBx;
	Prefs* prefs = NULL;

	switch (event->eType) {
	case frmOpenEvent:
		FrmDrawForm(form);

		prefs = GetPrefs();

		FrmSetFocus( form, FrmGetObjectIndex( form, qtTestField ) );
		SetCalView( prefs );

		autoSpaceChkBx 
			= FrmGetObjectPtr( form,
							   FrmGetObjectIndex( form, qtAutoSpaceChkBx ) );
		CtlSetValue( autoSpaceChkBx, prefs->autoSpace );

		FrmShowObject( form, FrmGetObjectIndex( form, qtLetterLocation ) );

		return true;
	case frmCloseEvent:
		Finished();
		break;

	case ctlSelectEvent:
		prefs = GetPrefs();
		switch (event->data.ctlSelect.controlID) {
		case qtYMinusButton:
			prefs->calY--;
			ensureValid( prefs );
			SetCalView( prefs );
			StorePrefs();
			return true;
		case qtYPlusButton:
			prefs->calY++;
			ensureValid( prefs );
			SetCalView( prefs );
			StorePrefs();
			return true;
		case qtXMinusButton:
			prefs->calX--;
			ensureValid( prefs );
			SetCalView( prefs );
			StorePrefs();
			return true;
		case qtXPlusButton:
			prefs->calX++;
			ensureValid( prefs );
			SetCalView( prefs );
			StorePrefs();
			return true;
		case qtAutoSpaceChkBx:
			autoSpaceChkBx
				= FrmGetObjectPtr( form,
								   FrmGetObjectIndex( form, 
													  qtAutoSpaceChkBx ) );
			prefs->autoSpace = CtlGetValue( autoSpaceChkBx );
			StorePrefs();
			return true;
		case qtCenterButton:
			prefs->calX = 0;
			prefs->calY = 0;
			SetCalView( prefs );
			StorePrefs();
			return true;
		case qtDoneButton:
			Finished();
			FrmGotoForm(hackMasterForm);
			return true;

		case qtAssignKeys:
			Finished();
			FrmGotoForm( qtKeyAssignForm );
   			return true;
		}
	}
	return false;
}

static void Finished() {
	Prefs* prefs = NULL;

	if( FtrGet( creatorId, qtPrefInfo, (DWord*) &prefs ) != ftrErrNoSuchFeature ) {
		MemPtrFree( prefs );
		FtrUnregister( creatorId, qtPrefInfo );
	}
}


static Prefs* GetPrefs() {
	Prefs* prefs = NULL;

	if( FtrGet( creatorId, qtPrefInfo, (DWord*) &prefs ) == ftrErrNoSuchFeature ) {
		UInt16 prefSize;

		prefs = MemPtrNew( sizeof( Prefs ) );
		prefs->autoSpace = false;
		prefs->calX = 0;
		prefs->calY = 0;

		// Leaves prefs set to defaults set above if does not exist
		PrefGetAppPreferences( creatorId, prfPrefs, prefs, &prefSize, true );

		// Make the pointer available
		FtrSet( creatorId, qtPrefInfo, (DWord) prefs );
	}

	ensureValid( prefs );

	return prefs;
}


static void StorePrefs() {
	Prefs* prefs;
	
	if( FtrGet( creatorId, qtPrefInfo, (DWord*) &prefs ) != ftrErrNoSuchFeature ) {
		PrefSetAppPreferences( creatorId, 
							   prfPrefs, 
							   0, 
							   prefs,
							   sizeof( Prefs ), 
							   true);
	}
}


static void ensureValid( Prefs* prefs ) {
	prefs->calX=(prefs->calX > qtMaxCalAdjust) ? qtMaxCalAdjust : prefs->calX;
	prefs->calY=(prefs->calY > qtMaxCalAdjust) ? qtMaxCalAdjust : prefs->calY;
	prefs->calX=(prefs->calX < -qtMaxCalAdjust)? -qtMaxCalAdjust : prefs->calX;
	prefs->calY=(prefs->calY < -qtMaxCalAdjust)? -qtMaxCalAdjust : prefs->calY;
}

static void SetCalView( Prefs* prefs ) {
	UInt16 index, centerIdx;
	FormPtr frm = FrmGetActiveForm();
	SWord centerX, centerY;

	index = FrmGetObjectIndex(frm, qtLetterLocation);
	centerIdx = FrmGetObjectIndex(frm, qtCenterLocation);

	FrmGetObjectPosition( frm, centerIdx, &centerX, &centerY );
	centerX += 1;
	FrmHideObject( frm, index );
	FrmSetObjectPosition( frm, index, centerX + prefs->calX, centerY + prefs->calY );
	FrmShowObject( frm, centerIdx );
	FrmShowObject( frm, index );
}
