#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include <Hardware/Hardware.h>

#include "quicktype.h"

DmOpenRef OpenOrCreateDB( Boolean* created );
void InitializeDB( DmOpenRef gDB );
void loadAndDisplay( UInt16 index );
void display( KeyEntry* entry );
void saveCurrent();
FieldPtr SetFieldTextFromStr(Word fieldID, Char* strP);
FieldPtr SetFieldTextFromHandle(Word fieldID, Handle txtH);
void PackKeyEntry( KeyEntry* entry, VoidHand record );
void PopulateFromDB( FormPtr form, DmOpenRef gDB );
VoidHand GetFieldTextHandle( UInt16 fieldId );


Boolean HandleEvent( EventPtr event ) {
	FormPtr form = FrmGetActiveForm();
	DmOpenRef gDB;
	Boolean created = false;

	switch (event->eType) {
	case frmOpenEvent:
		// open db
		FrmDrawForm(form);
		gDB = OpenOrCreateDB( &created );
		//if( created )
		//	InitializeDB( gDB );
		PopulateFromDB( form, gDB );
		FtrSet( creatorId, ftrGDB, (DWord) gDB );
		return true;

	case frmCloseEvent:
		if( FtrGet( creatorId, ftrGDB, (DWord*) &gDB ) == 0 ) {
			DmCloseDatabase( gDB );
			FtrUnregister( creatorId, ftrGDB );
			FtrUnregister( creatorId, ftrCurrentRecord );
		}
		break;

	case ctlSelectEvent:
		if( event->data.ctlSelect.controlID >= qtKeyAssign00Trigger
			&& event->data.ctlSelect.controlID <= qtKeyAssign34Trigger ) {
			UInt16 index = event->data.ctlSelect.controlID - qtKeyAssign00Trigger;
			loadAndDisplay( index );
			FrmSetFocus( form, FrmGetObjectIndex( form, qtKeyAssignPrimaryField ) );
			FtrSet( creatorId, ftrCurrentRecord, index );
			return true;
		}
		else if( event->data.ctlSelect.controlID == qtKeyAssignSave ) {
			UInt32 test;
			if( FtrGet( creatorId, ftrCurrentRecord, &test ) != ftrErrNoSuchFeature )
				saveCurrent();
			return true;
		}
		else if( event->data.ctlSelect.controlID == qtKeyAssignDone ) {
			// close db
			FtrGet( creatorId, ftrGDB, (DWord*) &gDB );
			DmCloseDatabase( gDB );
			FtrUnregister( creatorId, ftrGDB );
			FtrUnregister( creatorId, ftrCurrentRecord );
			FrmGotoForm(qtConfigForm);
			return true;
		}
	}
	return false;
}

DmOpenRef OpenOrCreateDB( Boolean* created ) {
	DmOpenRef gDB 
		= DmOpenDatabaseByTypeCreator('data', creatorId, dmModeReadWrite);
	Err err;

	if (!gDB) {
		VoidHand imageHandle = DmGetResource('DBIM', databaseImage);
		*created = true;
   
		DmCreateDatabaseFromImage(MemHandleLock(imageHandle));
		MemHandleUnlock(imageHandle);
		DmReleaseResource(imageHandle);

		gDB = DmOpenDatabaseByTypeCreator('data', creatorId, dmModeReadWrite);
	}
	return gDB;
}

/*
void InitializeDB( DmOpenRef gDB ) {
	KeyEntry entry = { '-', "", "" };
	UInt16 index, i;
	UInt16 numEntries = qtVertBoxes * ( qtHorizBoxes - 2 ); // minus two end columns
	for( i = 0; i < numEntries; i++ ) {
		UInt  index = dmMaxRecordIndex;
		VoidHand h = DmNewRecord( gDB, &index, 1 );
		if (h) {
			PackKeyEntry( &entry, h );
			DmReleaseRecord( gDB, index, true );
		}
	}
}
*/

void setTriggerToValue( FormPtr form, UInt16 controlId, Char value ) {
	ControlPtr trigger;
	CharPtr str;
	UInt16 controlIndex;

	controlIndex = FrmGetObjectIndex( form, controlId );
	trigger = FrmGetObjectPtr( form, controlIndex );

	// set control value
	FrmHideObject(form, controlIndex );
	str = CtlGetLabel( trigger );
	*str = value;
	FrmShowObject(form, controlIndex );
}

void PopulateFromDB( FormPtr form, DmOpenRef gDB ) {
	UInt16 index = 0;
	UInt16 controlId = qtKeyAssign00Trigger;
	VoidHand recordHandle;
	KeyEntry entry;
	UInt16 numRecords = DmNumRecords( gDB );

	for( index = 0; index < numRecords; index++, controlId++ ) {
		// Load record
		recordHandle = DmQueryRecord(gDB, index);

		UnpackKeyEntry( &entry, MemHandleLock( recordHandle ) );

		setTriggerToValue( form, controlId, entry.primary );
		
		MemHandleUnlock( recordHandle );
	}
}


void loadAndDisplay( UInt16 index ) {
	VoidHand myRecord = NULL;
	DmOpenRef gDB = NULL;
	KeyEntry entry;

	entry.primary = '\0';
	entry.left = "";
	entry.down = "";

	FtrGet( creatorId, ftrGDB, (DWord*) &gDB );
	// Load record
	myRecord = DmQueryRecord( gDB, index );

	if( myRecord != 0 )
		UnpackKeyEntry( &entry, MemHandleLock(myRecord) );
	
	// display record
	display( &entry );
	setTriggerToValue( FrmGetActiveForm(), qtKeyAssign00Trigger + index, entry.primary );

	if( myRecord != 0 )
		MemHandleUnlock( myRecord );
}

void display( KeyEntry* entry ) {
	Char primary[ 2 ];
	primary[0] = entry->primary;
	primary[1] = '\0';

	SetFieldTextFromStr( qtKeyAssignPrimaryField, primary );
	SetFieldTextFromStr( qtKeyAssignLeftField, entry->left );
	SetFieldTextFromStr( qtKeyAssignDownField, entry->down );
}

void saveCurrent() {
	DWord index;
	KeyEntry entry;
	FormPtr form = FrmGetActiveForm();
	VoidHand primaryH = GetFieldTextHandle( qtKeyAssignPrimaryField );
	VoidHand leftH = GetFieldTextHandle( qtKeyAssignLeftField );
	VoidHand downH = GetFieldTextHandle( qtKeyAssignDownField );
	VoidHand record;
	CharPtr primary;
	DmOpenRef gDB;

	// get current index
	FtrGet( creatorId, ftrCurrentRecord, &index );
	FtrGet( creatorId, ftrGDB, (DWord*) &gDB );

	entry.primary = *( (CharPtr) MemHandleLock( primaryH ) );
	entry.left = MemHandleLock( leftH );
	entry.down = MemHandleLock( downH );

	// Store current
	record = DmGetRecord( gDB, index );
	if( record ) {
		PackKeyEntry( &entry, record );
		DmReleaseRecord( gDB, index, true );
	}

	MemHandleUnlock( primaryH );
	MemHandleUnlock( leftH );
	MemHandleUnlock( downH );

	setTriggerToValue( form, qtKeyAssign00Trigger + index, entry.primary );
}


VoidHand GetFieldTextHandle( UInt16 fieldId ) {
	FormPtr form = FrmGetActiveForm();
	UInt16 fieldIndex;
	FieldPtr fieldPtr;

	fieldIndex = FrmGetObjectIndex( form, fieldId );
	fieldPtr = FrmGetObjectPtr( form, fieldIndex );
	return FldGetTextHandle( fieldPtr );
}


void PackKeyEntry( KeyEntry* entry, VoidHand record ) {
	UInt16 length = 0;
	CharPtr s;
	UInt16 offset = 0;

	length = 1 // primary
		+ StrLen( entry->left ) + 1 // terminator
		+ StrLen( entry->down ) + 1; // terminator
      
	// resize the VoidHand
	if( MemHandleResize( record, length ) == 0 ) {
		// copy the fields
		s = MemHandleLock( record );
		offset = 0;
		DmWrite( s, offset, (CharPtr) &entry->primary, sizeof( entry->primary ) );
		offset += sizeof( entry->primary );
		DmStrCopy( s, offset, (CharPtr) entry->left );
		offset += StrLen( entry->left ) + 1;
		DmStrCopy( s, offset, (CharPtr) entry->down );
		MemHandleUnlock( record );
	}
	else {
		Err err;
		char buffer[ 50 ];
		err = DmGetLastError();
		StrIToH( buffer, err );
		WinDrawChars( buffer, StrLen( buffer ), 10, 10 );
	}
}

// lock record before, and unlock after.  free memory for keyentry.
void UnpackKeyEntry( KeyEntry* entry, const VoidPtr record ) {
	CharPtr s = record;
	UInt16 i = 0;
	UInt16 offset;
	UInt16 len;

	offset = 0;
	entry->primary = *s;
	offset = 1;
	entry->left = s + offset;
	offset += StrLen( entry->left ) + 1;
	entry->down = s + offset;
}


FieldPtr SetFieldTextFromStr(Word fieldID, Char* strP) {
   VoidHand txtH;
   
   // get some space in which to stash the string.
   txtH  = MemHandleNew(StrLen(strP) + 1);
   if (!txtH)
      return NULL;

   // copy the string to the locked handle.
   StrCopy(MemHandleLock(txtH), strP);

   // unlock the string handle.
   MemHandleUnlock(txtH);
   
   // set the field to the handle
   return (FieldPtr) SetFieldTextFromHandle(fieldID, (Handle) txtH);
} 

FieldPtr SetFieldTextFromHandle(Word fieldID, Handle txtH) {
	Handle oldTxtH;
	FormPtr     frm = FrmGetActiveForm();
	FieldPtr    fldP;
	

	// get the field and the field's current text handle.
	fldP     = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
	ErrNonFatalDisplayIf(!fldP, "missing field");
	oldTxtH  = FldGetTextHandle(fldP);
   
	// set the field's text to the new text.
	FldSetTextHandle(fldP, txtH);
	FldDrawField(fldP);

	// free the handle AFTER we call FldSetTextHandle().
	if (oldTxtH) 
		MemHandleFree( (VoidHand) oldTxtH);
	
	return fldP;
}  
