
/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author
 * @version 1.0
 */
package textcounter;

import java.util.*;
import java.io.*;

public class TextCounter {
  private static final ReverseOrderComparator comp = new ReverseOrderComparator();
  HashMap entries = new HashMap();
  Entry lastEntry;
  File storeFile = new File( "c:/temp/entries.ser" );
  private boolean showCount = false;

  public TextCounter() {
  }

  private void readFile( File fileIn ) throws IOException {
    FileReader fr = null;
    BufferedReader br = null;
    try {
      fr = new FileReader( fileIn );
      br = new BufferedReader( fr );
      String line = null;
      while( br.ready() ) {
        line = br.readLine();
        addString( line );
        System.out.print( '.' );
      }
      System.out.println();
    }
    catch( EOFException e ){}
    finally {
      if( br != null ) { try { br.close(); } catch( Exception e ){} }
      if( fr != null ) { try { fr.close(); } catch( Exception e ){} }
    }
  }

  private void addString( String s ) {
    char[] characters = s.toCharArray();
    Entry entry = null;
    Character ch = null;
    for( int i = 0; i < characters.length; i++ ) {
      if( ! Character.isLetter( characters[ i ] ) && ! ( characters[ i ] == ' ' ) )
        continue;
      characters[ i ] = Character.toUpperCase(characters[ i ]);
      ch = new Character( characters[ i ] );
      entry = (Entry) entries.get( ch );
      if( entry == null )
        entry = new Entry( ch );
      entry.increment();
      if( lastEntry != null && ! ( lastEntry.equals(entry) ) ) {
        entry.addNeighbor( lastEntry );
        lastEntry.addNeighbor(entry);
      }
      entries.put( ch, entry );
      lastEntry = entry;
    }
  }

  public void store() throws IOException {
      FileOutputStream fout = null;
      ObjectOutputStream oout = null;
    try {
      fout = new FileOutputStream( storeFile );
      oout = new ObjectOutputStream( fout );
      oout.writeObject(entries);
      oout.writeObject(lastEntry);
    }
    finally {
      if( oout != null ) { try { oout.close(); } catch( Exception e ){} }
      if( fout != null ) { try { fout.close(); } catch( Exception e ){} }
    }
  }

  public void load() throws IOException {
    FileInputStream fin = null;
    ObjectInputStream oin = null;
    try {
      fin = new FileInputStream( storeFile );
      oin = new ObjectInputStream( fin );
      entries = (HashMap) oin.readObject();
      lastEntry = (Entry) oin.readObject();
    }
    catch( ClassNotFoundException e ) { e.printStackTrace(); }
    finally {
      if( oin != null ) { try { oin.close(); } catch( Exception e ){} }
      if( fin != null ) { try { fin.close(); } catch( Exception e ){} }
    }
  }

  public void runReport() {
    try {
      load();
      Set set = getSortedEntries();
      Entry entry = null;
      for( Iterator i = set.iterator(); i.hasNext(); ) {
        entry = (Entry) i.next();
        System.out.print( entry );
        if( showCount ) {
          System.out.print( " " + entry.getCount() );
        }
        System.out.println();
      }
    }
    catch( Exception e ) {
      e.printStackTrace();
    }
  }

  private Set getSortedEntries() {
    TreeSet set = new TreeSet(comp);
    set.addAll( entries.values() );
    return set;
  }

  public void showCount( boolean show ) {
    this.showCount = show;
  }

  public void execute( File inputFile ) {
    try {
      load();
    }
    catch( IOException e ) {
      System.out.println( "running for first" );
    }
    try {
      readFile( inputFile );
    }
    catch( IOException e ) {
      e.printStackTrace();
    }
    finally {
      try {
        store();
      }
      catch( IOException e ) {
        e.printStackTrace();
      }
    }
  }

  public static void main(String[] args) {
    TextCounter textCounter1 = new TextCounter();

    if( args.length == 0 ) {
      System.out.println( "-p  print report" );
      System.out.println( "-pc print report with count" );
    }

    for( int argNum = 0; argNum < args.length; argNum++ ) {
      if( args[ argNum ].equals( "-p" ) ) {
        textCounter1.runReport();
      }
      else if( args[ argNum ].equals( "-pc" ) ) {
        textCounter1.showCount( true );
        textCounter1.runReport();
      }
      else {
        File file = new File( args[ 0 ] );
        textCounter1.execute( file );
      }
    }
  }
}

